package org.seasar.velocity.tools;

import org.apache.velocity.tools.view.servlet.ServletToolInfo;
import org.apache.velocity.tools.view.tools.ViewTool;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.framework.util.ClassUtil;

public class S2ServletToolInfo extends ServletToolInfo {

  protected Class   clazz;
  protected boolean initializable;

  /**
   * このコードを大幅に変更した。
   * ===
   * Returns a new instance of the tool. If the tool
   * implements {@link ViewTool}, the new instance
   * will be initialized using the given data.
   */
  public Object getInstance(Object initData) {
    Object tool = null;

    S2Container container = SingletonS2ContainerFactory.getContainer();
    if (container.hasComponentDef(clazz))
      tool = container.getComponent(clazz);
    else
      tool = ClassUtil.newInstance(clazz);

    if (initializable) {
      ((ViewTool) tool).init(initData);
    }
    return tool;
  }

  /**
   * If an instance of the tool cannot be created from
   * the classname passed to this method, it will throw an exception.
   *
   * @param classname the fully qualified java.lang.Class of the tool
   */
  public void setClassname(String classname) throws Exception {
    this.clazz = getApplicationClass(classname);
    /* create an instance and see if it is initializable */
    if (hasViewToolInterface(this.clazz)) {
      this.initializable = true;
    }
  }

  /**
   * コンストラクタに引数がある場合、S2コンテナを通してインスタンス生成
   * するようにしたいため、Class#newInstance() を安易に使えない。
   * そのため、以下のようにして ViewToolインタフェースの実装有無を確かめる
   * 必要がある。
   */
  private boolean hasViewToolInterface(Class clazz)
      throws InstantiationException, IllegalAccessException {
    if (clazz == Object.class)
      return false;
    else {
      for (int i = 0; i < clazz.getInterfaces().length; i++)
        if (clazz.getInterfaces()[i] == ViewTool.class) return true;
      return hasViewToolInterface(clazz.getSuperclass());
    }

  }

  /**
   * Return the <code>Class</code> object for the specified fully qualified
   * class name, from this web application's class loader.  If no 
   * class loader is set for the current thread, then the class loader
   * that loaded this class will be used.
   *
   * @param name Fully qualified class name to be loaded
   * @return Class object
   * @exception ClassNotFoundException if the class cannot be found
   * @since VelocityTools 1.1
   */
  protected Class getApplicationClass(String name)
      throws ClassNotFoundException {
    ClassLoader loader = Thread.currentThread().getContextClassLoader();
    if (loader == null) {
      loader = S2ServletToolInfo.class.getClassLoader();
    }
    return loader.loadClass(name);
  }

  public String getClassname() {
    return clazz.getName();
  }
}